@@ -6,13 +6,16 @@
 #endif
 #include "mainwindow.h"
 #include "optiondialog.h"
+#include "equalizerdialog.h"
 #include "time.h"
 
 //#define AVOID_INPUT_DIALOG
 
 MainWindow::MainWindow()
-    : plman (this), settings (tr ("TomAmp"), "TomAmp"), isPlaying (false), autostopped (false)
+    : eqEffect (0), plman (this), settings (tr ("TomAmp"), "TomAmp"), isPlaying (false), autostopped (false)
 {
+    QList<Phonon::EffectDescription> effectDescriptions =
+                 Phonon::BackendCapabilities::availableAudioEffects();
     audioOutput = new Phonon::AudioOutput(Phonon::MusicCategory, this);
     mediaObject = new Phonon::MediaObject(this);
 
@@ -28,11 +31,38 @@ MainWindow::MainWindow()
     connect (&plman, SIGNAL (itemRemoved(int)), this, SLOT (itemRemoved (int)));
 
     connect (QApplication::desktop(), SIGNAL(resized(int)), this, SLOT(orientationChanged()));
+    timer.setSingleShot(true);
+    connect (&timer, SIGNAL(timeout()), this, SLOT(hideVolumeBar()));
 #ifdef Q_WS_MAEMO_5
     QDBusConnection::systemBus().connect(QString(), "/org/freedesktop/Hal/devices/platform_headphone", "org.freedesktop.Hal.Device", "Condition", this, SLOT(headphoneEvent(QString,QString)));
 #endif    
+    foreach (Phonon::EffectDescription effectDescription, effectDescriptions) 
+    {
+//        qDebug () << effectDescription.name();
+        if ((effectDescription.name() == "KEqualizer") || (effectDescription.name() == "equalizer-10bands"))
+        {
+/*            foreach (QString str, effectDescription.propertyNames())
+            {
+                qDebug() << str << " => " << effectDescription.property(str.toAscii());
+            }*/
+            eqEffect = new Phonon::Effect(effectDescription);
+//            qDebug() << "Effect: " << eqEffect;
+            int i = 0;
+            foreach (Phonon::EffectParameter par, eqEffect->parameters())
+            {
+                QString valName = QString ("eq_%1").arg(i++);
+                double dval = settings.value(valName, 0.0).toDouble();
+                eqEffect->setParameterValue(par, dval);
+//                qDebug() << par.name() << " => " << par.minimumValue() << " to " << par.maximumValue();
+            }
+        }
+    }
 
-    Phonon::createPath(mediaObject, audioOutput);
+    Phonon::Path path = Phonon::createPath(mediaObject, audioOutput);
+    if (eqEffect)
+    {
+        path.insertEffect(eqEffect);
+    }
 
     qsrand (time (NULL));
     repeat = settings.value("repeat", false).toBool();
@@ -62,8 +92,8 @@ MainWindow::MainWindow()
     int curind = settings.value("currentIndex", -1).toInt ();
     if (curind >= 0)
         setItem (curind, false);
-    audioOutput->setVolume(settings.value("volume", .5).toReal());
-    QApplication::setWindowIcon(QIcon (QPixmap (":images/tomamp")));
+    audioOutput->setVolume(settings.value("volume", .5).toDouble());
+    QApplication::setWindowIcon(QIcon (QPixmap (":images/Tomamp.png")));
     setOrientation();
 }
 
@@ -73,7 +103,7 @@ MainWindow::~MainWindow()
     settings.setValue("repeat", repeat);
     settings.setValue("lastPlaylist", plman.playlistStrings());
     settings.setValue("volume", audioOutput->volume());
-    settings.setValue("currentIndex", plman.indexOf(mediaObject->currentSource()));
+    settings.setValue("currentIndex", lastPlayed);
     settings.setValue("headers", headers);
     for (int i = 0; i < musicTable->columnCount(); ++i)
     {
@@ -109,7 +139,11 @@ void MainWindow::orientationChanged()
     setupUi();
     int current = musicTable->currentRow();
     playlistChanged(0);
-    musicTable->selectRow(current);
+    if (current >= 0)
+    {
+        musicTable->selectRow(current);
+        highlightRow(current);
+    }
 }
 
 
@@ -182,9 +216,9 @@ void MainWindow::addUrl()
 void MainWindow::about()
 {
     QMessageBox msg;
-    msg.setIconPixmap(QPixmap (":images/tomamp"));
+    msg.setIconPixmap(QPixmap (":images/Tomamp.png"));
     msg.setModal(true);
-    msg.setWindowTitle(tr ("About TomAmp v0.3"));
+    msg.setWindowTitle(tr ("About TomAmp v0.4"));
     msg.setText(tr("TomAmp is a simple playlist-based music player.\n\n"
                 "(c) 2010 Tamas Marki <tmarki@gmail.com>\n\n"
                 "Please send comments and bug reports to the above e-mail address.\n\n"
@@ -212,8 +246,11 @@ void MainWindow::stateChanged(Phonon::State newState, Phonon::State /* oldState
             break;
         case Phonon::PlayingState:
             setWindowTitle(mediaObject->metaData().value("TITLE") + "(" + mediaObject->metaData().value("ARTIST") + ") - TomAmp");
-            pauseAction->setVisible(true);
-            playAction->setVisible (false);
+            if (stopAction->isVisible())
+            {            
+                pauseAction->setVisible(true);
+                playAction->setVisible (false);
+            }
             playAction->setEnabled(false);
             pauseAction->setEnabled(true);
             stopAction->setEnabled(true);
@@ -227,8 +264,11 @@ void MainWindow::stateChanged(Phonon::State newState, Phonon::State /* oldState
             setWindowTitle("TomAmp");
             stopAction->setEnabled(false);
             playAction->setEnabled(true);
-            pauseAction->setVisible(false);
-            playAction->setVisible(true);
+            if (stopAction->isVisible())
+            {            
+                pauseAction->setVisible(false);
+                playAction->setVisible(true);
+            }
             pauseAction->setEnabled(false);
             timeLcd->display("00:00:00");
             unhighlightRow(plman.indexOf(mediaObject->currentSource()));
@@ -237,8 +277,11 @@ void MainWindow::stateChanged(Phonon::State newState, Phonon::State /* oldState
         case Phonon::PausedState:
             pauseAction->setEnabled(false);
             stopAction->setEnabled(true);
-            pauseAction->setVisible(false);
-            playAction->setVisible(true);
+            if (stopAction->isVisible())
+            {            
+                pauseAction->setVisible(false);
+                playAction->setVisible(true);
+            }
             playAction->setEnabled(true);
             break;
         case Phonon::BufferingState:
@@ -506,42 +549,48 @@ void MainWindow::finished()
 
 void MainWindow::setupActions()
 {
-    playAction = new QAction(QIcon (QPixmap (":images/play")), "", this);
+    playAction = new QAction(QIcon (QPixmap (":images/play.png")), "", this);
     playAction->setShortcut(tr("Crl+P"));
     playAction->setDisabled(true);
-    pauseAction = new QAction(QIcon (QPixmap (":images/pause")), "", this);
+    pauseAction = new QAction(QIcon (QPixmap (":images/pause.png")), "", this);
     pauseAction->setShortcut(tr("Ctrl+A"));
     pauseAction->setDisabled(true);
     pauseAction->setVisible(false);
-    stopAction = new QAction(QIcon (QPixmap (":images/stop")), "", this);
+    stopAction = new QAction(QIcon (QPixmap (":images/stop.png")), "", this);
     stopAction->setShortcut(tr("Ctrl+S"));
     stopAction->setDisabled(true);
-    nextAction = new QAction(QIcon (QPixmap (":images/next")), "", this);
+    nextAction = new QAction(QIcon (QPixmap (":images/next.png")), "", this);
     nextAction->setShortcut(tr("Ctrl+N"));
 //    upAction = new QAction (QString::fromUtf8("▲"), this);
 //    downAction = new QAction (QString::fromUtf8("▼"), this);
 //    delAction = new QAction (QString::fromUtf8("╳"), this);
 //    enqueueActionButton = new QAction (tr ("E"), this);
-    upAction = new QAction(QIcon (QPixmap (":images/up")), "", this);
-    downAction = new QAction(QIcon (QPixmap (":images/down")), "", this);
-    delAction = new QAction(QIcon (QPixmap (":images/remove")), "", this);
-    addAction = new QAction(QIcon (QPixmap (":images/add")), "", this);
-    enqueueActionButton = new QAction(QIcon (QPixmap (":images/enqueue")), "", this);
-    previousAction = new QAction(QIcon (QPixmap (":images/previous")), "", this);
+    upAction = new QAction(QIcon (QPixmap (":images/up.png")), "", this);
+    upAction->setVisible(false);
+    downAction = new QAction(QIcon (QPixmap (":images/down.png")), "", this);
+    downAction->setVisible(false);
+    delAction = new QAction(QIcon (QPixmap (":images/remove.png")), "", this);
+    delAction->setVisible(false);
+    addAction = new QAction(QIcon (QPixmap (":images/add.png")), "", this);
+    addAction->setVisible(false);
+    enqueueActionButton = new QAction(QIcon (QPixmap (":images/enqueue.png")), "", this);
+    enqueueActionButton->setVisible(false);
+    previousAction = new QAction(QIcon (QPixmap (":images/previous.png")), "", this);
     previousAction->setShortcut(tr("Ctrl+R"));
     if (repeat)
-        repeatAction = new QAction(QIcon (QPixmap (":images/repeat")), "", this);
+        repeatAction = new QAction(QIcon (QPixmap (":images/repeat.png")), "", this);
     else
-        repeatAction = new QAction(QIcon (QPixmap (":images/repeat_off")), "", this);
+        repeatAction = new QAction(QIcon (QPixmap (":images/repeat_disabled.png")), "", this);
     repeatAction->setShortcut(tr("Ctrl+I"));
     if (shuffle)
-        shuffleAction = new QAction(QIcon (QPixmap (":images/shuffle")), "", this);
+        shuffleAction = new QAction(QIcon (QPixmap (":images/shuffle.png")), "", this);
     else
-        shuffleAction = new QAction(QIcon (QPixmap (":images/shuffle_off")), "", this);
+        shuffleAction = new QAction(QIcon (QPixmap (":images/shuffle_disabled.png")), "", this);
     shuffleAction->setShortcut(tr("Ctrl+H"));
-    volumeAction = new QAction(QIcon (QPixmap (":images/volume")), "", this);
+    volumeAction = new QAction(QIcon (QPixmap (":images/volume.png")), "", this);
+    volumeAction->setVisible(false);
     volumeAction->setCheckable(true);
-    volumeAction->setShortcut(tr("Ctrl+V"));
+    toggleControls = new QAction(QIcon (QPixmap (":images/arrow1.png")), "", this);
     addFilesAction = new QAction(tr("Add &File"), this);
     addFilesAction->setShortcut(tr("Ctrl+F"));
     addFoldersAction = new QAction(tr("Add F&older"), this);
@@ -556,6 +605,8 @@ void MainWindow::setupActions()
     clearPlaylistAction->setShortcut(tr("Ctrl+C"));
     optionAction = new QAction (tr("Op&tions"), this);
     optionAction->setShortcut(tr("Ctrl+T"));
+    equalizerAction = new QAction (tr("&Equalizer"), this);
+    equalizerAction->setShortcut(tr("Ctrl+E"));
     exitAction = new QAction(tr("E&xit"), this);
     exitAction->setShortcut(tr("Ctrl+X"));
     aboutAction = new QAction(tr("A&bout"), this);
@@ -573,6 +624,7 @@ void MainWindow::setupActions()
     connect(volumeAction, SIGNAL(triggered()), this, SLOT(volumeToggle()));
     connect(enqueueActionButton, SIGNAL(triggered()), this, SLOT(enqueueSelected()));
     connect (addAction, SIGNAL(triggered()), this, SLOT(showAddMenu()));
+    connect(toggleControls, SIGNAL(triggered()), this, SLOT(controlsToggled()));
 
     connect(addFilesAction, SIGNAL(triggered()), this, SLOT(addFiles()));
     connect(addFoldersAction, SIGNAL(triggered()), this, SLOT(addFolder()));
@@ -581,6 +633,7 @@ void MainWindow::setupActions()
     connect (loadPlaylistAction, SIGNAL (triggered()), this, SLOT (loadPlaylist()));
     connect (clearPlaylistAction, SIGNAL (triggered()), &plman, SLOT (clearPlaylist()));
     connect (optionAction, SIGNAL (triggered()), this, SLOT (showOptions()));
+    connect (equalizerAction, SIGNAL (triggered()), this, SLOT (showEqualizer()));
     connect (nextAction, SIGNAL(triggered()), this, SLOT(next()));
     connect (previousAction, SIGNAL(triggered()), this, SLOT(previous()));
     connect (upAction, SIGNAL(triggered()), this, SLOT(upSelected()));
@@ -621,9 +674,9 @@ void MainWindow::repeatToggle ()
     repeat = !repeat;
     settings.setValue("repeat", QVariant (repeat));
     if (repeat)
-        repeatAction->setIcon(QIcon (QPixmap (":images/repeat")));
+        repeatAction->setIcon(QIcon (QPixmap (":images/repeat.png")));
     else
-        repeatAction->setIcon(QIcon (QPixmap (":images/repeat_off")));
+        repeatAction->setIcon(QIcon (QPixmap (":images/repeat_disabled.png")));
 }
 
 void MainWindow::shuffleToggle ()
@@ -631,18 +684,23 @@ void MainWindow::shuffleToggle ()
     shuffle = !shuffle;
     settings.setValue("shuffle", QVariant (shuffle));
     if (shuffle)
-        shuffleAction->setIcon(QIcon (QPixmap (":images/shuffle")));
+        shuffleAction->setIcon(QIcon (QPixmap (":images/shuffle.png")));
     else
-        shuffleAction->setIcon(QIcon (QPixmap (":images/shuffle_off")));
+        shuffleAction->setIcon(QIcon (QPixmap (":images/shuffle_disabled.png")));
 }
 
 void MainWindow::volumeToggle ()
 {
     volumeSlider->setVisible(volumeAction->isChecked());
-    connect (&timer, SIGNAL(timeout()), volumeSlider, SLOT(hide()));
-    timer.setSingleShot(true);
-    timer.start(3000);
-//    seekSlider->setVisible(!volumeAction->isChecked());
+    musicTable->show();
+    timer.start(5000);
+}
+
+void MainWindow::hideVolumeBar()
+{
+    volumeSlider->hide();
+    musicTable->show();
+    volumeAction->setChecked(false);
 }
 
 void MainWindow::play()
@@ -671,7 +729,8 @@ void MainWindow::setupMenus()
     fileMenu->addAction(loadPlaylistAction);
     fileMenu->addAction(clearPlaylistAction);
     fileMenu->addAction(optionAction);
-    fileMenu->addAction(exitAction);
+    fileMenu->addAction(equalizerAction);
+//    fileMenu->addAction(exitAction);
 
     QMenu *aboutMenu = menuBar()->addMenu(tr("&Help"));
     aboutMenu->addAction(aboutAction);
@@ -702,25 +761,22 @@ void MainWindow::setupUi()
         volumeSlider->setMinimumWidth(150);
     }
     volumeSlider->setMuteVisible(false);
+    int iconsize = settings.value ("iconsize", 48).toInt();
+    bar->setIconSize(QSize (iconsize, iconsize));
+    bar->addAction(previousAction);
     bar->addAction(playAction);
     bar->addAction(pauseAction);
     bar->addAction(stopAction);
-    bar->addSeparator();
+    bar->addAction(nextAction);
     bar->addAction(repeatAction);
     bar->addAction(shuffleAction);
-    bar->addSeparator();
-    bar->addAction(previousAction);
-    bar->addAction(nextAction);
-    bar->addSeparator();
     bar->addAction(upAction);
     bar->addAction(downAction);
-    bar->addSeparator();
     bar->addAction(delAction);
     bar->addAction(addAction);
-    bar->addSeparator();
     bar->addAction(enqueueActionButton);
-    bar->addSeparator();
     bar->addAction(volumeAction);
+    bar->addAction(toggleControls);
 
     contextMenu = new QMenu (this);
     enqueueAction = contextMenu->addAction(tr ("Enqueue"));
@@ -764,7 +820,6 @@ void MainWindow::setupUi()
             musicTable->setColumnWidth(i, val);
     }
 
-
     QLayout *seekerLayout;
     QLayout *playbackLayout;
     if (flip)
@@ -818,7 +873,6 @@ void MainWindow::cellClicked(int /*row*/, int)
 void MainWindow::enqueueSelected()
 {
     int sel = musicTable->currentRow();
-    qDebug () << "Enqueue on " << sel;
     if (sel >= 0)
     {
         mediaObject->queue().clear();
@@ -919,7 +973,10 @@ void MainWindow::setRowFromItem (int row, const PlaylistItem& item)
         int col = headers.indexOf("Title");
         if (col >= 0)
         {
-            QTableWidgetItem *item1 = new QTableWidgetItem(item.uri);
+            QString title = item.uri;
+            QFileInfo fi (title);
+            title = fi.fileName();
+            QTableWidgetItem *item1 = new QTableWidgetItem(title);
             item1->setFlags(item1->flags() ^ Qt::ItemIsEditable);
             musicTable->setItem(row, col, item1);
         }
@@ -1103,22 +1160,49 @@ void MainWindow::downSelected()
 void MainWindow::showOptions ()
 {
     int oldsize = settings.value("rowHeight", DEFAULT_ROW_HEIGHT).toInt();
+    int oldiconsize = settings.value("iconsize", 48).toInt();
     OptionDialog* dlg = new OptionDialog (this, settings);
     dlg->exec();
     delete dlg;
+    if (oldiconsize != settings.value("iconsize", 48).toInt())
+    {
+        delete centralWidget();
+        setupUi();
+    }
     setOrientation ();
-    if (headers != settings.value("headers", QStringList ()).toStringList() || oldsize != settings.value("rowHeight", DEFAULT_ROW_HEIGHT).toInt())
+    if (headers != settings.value("headers", QStringList ()).toStringList()
+        || oldsize != settings.value("rowHeight", DEFAULT_ROW_HEIGHT).toInt()
+        || oldiconsize != settings.value("iconsize", 48).toInt())
     {
         headers = settings.value("headers", QStringList ()).toStringList();
         musicTable->setColumnCount(headers.size ());
         musicTable->setHorizontalHeaderLabels(headers);
+        int current = musicTable->currentRow();
         playlistChanged(0);
+        musicTable->selectRow(current);
     }
     int curitem = plman.indexOf(mediaObject->currentSource());
     if (curitem >= 0 && isPlaying)
         highlightRow(curitem);
 }
 
+void MainWindow::showEqualizer ()
+{
+    QList<int> vals;
+    if (eqEffect)
+    {
+        foreach(Phonon::EffectParameter ep, eqEffect->parameters())
+        {
+            vals.push_back(int (eqEffect->parameterValue(ep).toDouble() * 10.0));
+        }
+    }
+    EqualizerDialog* dlg = new EqualizerDialog (this, vals);
+    connect (dlg, SIGNAL(equalizerChanged(int,int)), this, SLOT(equalizerChanged(int,int)));
+    dlg->exec();
+    delete dlg;
+}
+
+
 void MainWindow::showAddMenu()
 {
     addMenu->popup(cursor().pos());
@@ -1142,3 +1226,54 @@ void MainWindow::headphoneEvent(QString what,QString how)
         autostopped = false;
     }
 }
+
+void MainWindow::equalizerChanged (int ind, int val)
+{
+    if (eqEffect)
+    {
+        double dval = (double)val / 10.0;
+        eqEffect->setParameterValue(eqEffect->parameters()[ind], dval);
+        QString valName = QString ("eq_%1").arg(ind);
+        settings.setValue(valName, dval);
+    }
+}
+
+void MainWindow::controlsToggled ()
+{
+    if (stopAction->isVisible())
+    {
+        playAction->setVisible(false);
+        pauseAction->setVisible(false);
+        stopAction->setVisible(false);
+        nextAction->setVisible(false);
+        previousAction->setVisible(false);
+        shuffleAction->setVisible(false);
+        repeatAction->setVisible(false);
+        upAction->setVisible(true);
+        downAction->setVisible(true);
+        delAction->setVisible(true);
+        addAction->setVisible(true);
+        enqueueActionButton->setVisible(true);
+        volumeAction->setVisible(true);
+        toggleControls->setIcon(QIcon (QPixmap (":images/arrow2.png")));
+    }
+    else
+    {
+        if (mediaObject->state() == Phonon::PlayingState)
+            pauseAction->setVisible(true);
+        else
+            playAction->setVisible(true);
+        stopAction->setVisible(true);
+        nextAction->setVisible(true);
+        previousAction->setVisible(true);
+        shuffleAction->setVisible(true);
+        repeatAction->setVisible(true);
+        upAction->setVisible(false);
+        downAction->setVisible(false);
+        delAction->setVisible(false);
+        addAction->setVisible(false);
+        enqueueActionButton->setVisible(false);
+        volumeAction->setVisible(false);
+        toggleControls->setIcon(QIcon (QPixmap (":images/arrow1.png")));
+    }
+}
